﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Loz;

namespace Testy
{
    [TestClass]
    public class TestySilnika
    {
        [TestMethod]
        /*
         * Metoda bada poprawną pozycję statku gracza względem wylosowanej pary (X, Y) skalującej ekran i inne
         * elementy wykonawcze. Test nie zostanie zaliczony, gdy statek wyjdzie swym ciałem za obszar dozwolony.
         */
        public void TestPozycjiStatkuGracza()
        {
            Silnik silnik = new Silnik();
            Random x = new Random();
            Random y = new Random();
            int poprawnePróby = 0;
            const int ilośćPrób = 50;
            silnik.Resetuj();
            for (int i = 0; i < ilośćPrób; i++)
            {
                silnik.PrzeskalujRozmiary(x.Next(100, 1920), y.Next(100, 1080));
                if (silnik.PrzestrzeńXpx >= silnik.ZwróćXStatku() && silnik.PrzestrzeńYpx >= silnik.ZwróćYStatku())
                {
                    poprawnePróby++;
                }
            }
            Assert.AreEqual(ilośćPrób, poprawnePróby);
        }
        [TestMethod]
        /*
         * Metoda bada poprawność przepływu informacji tyczących się skalowania ekranu i elementów roboczych.
         * N-ta ilość prób powoduje skalowanie ekranu, a następnie potwierdzenie otrzymania poprawnej informacji
         * o obecnej skali elementów poprzez wywołanie metody ZwróćXPlanszy, ZwróćYPlanszy (utworzone na potrzeby testów).
         */
        public void TestSkalowaniaEkranu()
        {
            Silnik silnik = new Silnik();
            int poprawnePróby = 0;
            const int ilośćPrób = 500;
            silnik.Resetuj();
            for(int i = 1; i <= ilośćPrób; i++) //dla i=0 zgłaszany wyjątek o niepoprawnej rozdzielczości ekranu.
            {
                silnik.PrzeskalujRozmiary(i, i);
                if(silnik.PrzestrzeńXpx == i && silnik.PrzestrzeńYpx == i)
                {
                    poprawnePróby++;
                }
            }
            Assert.AreEqual(ilośćPrób, poprawnePróby);
        }
        [TestMethod]
        /*
         * Metoda generująca wyłącznie jeden strzał.
         * Badane jest poprawne opuszczenie planszy przez pocisk po wykonaniu odpowiedniej ilości kroków.
         */
        public void TestPojedyńczegoStrzału()
        {
            Silnik silnik = new Silnik();
            const int ilośćKrokówDoZniknięciaStrzału = 22;
            silnik.Resetuj();
            silnik.NaciśniętoKlawiszSpace(); //wygenerowanie jednego strzału
            //22 jest wartością graniczną, do której to strzał gracza "istnieje" na planszy gry.
            //wartość ta została wyliczona metodą prób i błędów tego testu jednostkowego
            for (int i = 0; i < ilośćKrokówDoZniknięciaStrzału; i++) 
            {
                //wygenerowanie ruchu pocisków - bez tego pocisk nie poruszy się ku górze planszy.
                silnik.WykonajKrok();
            }
            //sprawdzanie czy pierwszy wygenerowany (przed pętlą) pocisk poprawnie zniknał z planszy.
            Assert.AreEqual(false, silnik.CzyPociskGraczaIstnieje(0));
        }
        /*
         * Metoda używana do testowania wielu strzałów (TestWieluStrzałów)
         * Zwraca true, gdy istnieje jakikolwiek strzał.
         * Zwraca false, gdy plansza jest pusta.
         */
        [TestMethod]
        private bool czyJakikolwiekStrzałIstnieje(Silnik silnik, int ilośćStrzałówJednocześnie)
        {
            int ilośćNieistniejących = 0;
            if (ilośćStrzałówJednocześnie <= 0) return false;
            for(int i = 0; i < ilośćStrzałówJednocześnie; i++)
            {
                if (silnik.CzyPociskGraczaIstnieje(i)) ilośćNieistniejących++;
            }
            return (ilośćNieistniejących == ilośćStrzałówJednocześnie) ? false : true;
        }
        /*Rozbudowana wersja TestPojedyńczegoStrzału
         * Zakłada istnienie kilku (<=maxIlośćStrzałów) pocisków jednocześnie
         * Metoda testowa bada czy wszystkie strzały poprawnie zakończyły swoje istnienie (tj. wyszły za plansze)
        */
        [TestMethod]
        public void TestWieluStrzałów()
        {
            Silnik silnik = new Silnik();
            Random budowanieOdległościMiędzyStrzałami = new Random();
            const int ilośćJednoczesnychStrzałów = 2;
            const int ilośćPrób = 5;
            int ilośćWykonanychPrób = 0;
            silnik.Resetuj();

            while(ilośćWykonanychPrób != (ilośćPrób - 1))
            {
                silnik.WykonajKrok();
                for(int i = 0; i < ilośćJednoczesnychStrzałów; i++)
                {
                    if(!silnik.CzyPociskGraczaIstnieje(i))
                    {
                        ilośćWykonanychPrób++;
                        /* WAŻNE!
                         * Nie ma możliwości, by podczas budowania odległości między strzałami
                         * pojawiła się sytuacja, w której to inny strzał przestanie "istnieć" i nie zostanie
                         * poprawnie obsłużony. Po zbudowaniu odległości, w kolejnej iteracji sprawdza się istnienie kolejnego elementu.
                         * Jeśli element przestał "istnieć" gdzieś pomiędzy rozpoczęciem budowania odległości, a końcem budowy, zostanie
                         * on obsłużony w kolejnej iteracji i życie rozpocznie nowy element.
                         */
                        for(int k = budowanieOdległościMiędzyStrzałami.Next(1, 5); k > 0; k--)
                        {
                            silnik.WykonajKrok();
                        }
                        silnik.NaciśniętoKlawiszSpace();
                    }
                }
            }
            //Poniższa pętla wykonuje tyle ruchów, by ostatni strzał mógł opuścić planszę.
            while(czyJakikolwiekStrzałIstnieje(silnik, ilośćJednoczesnychStrzałów) == true)
            {
                silnik.WykonajKrok();
            }
            ilośćWykonanychPrób++;
            Assert.AreEqual(ilośćWykonanychPrób, ilośćPrób);
        }
        /*Metoda sprawdza czy dla n-tej ilości przeciwników z m-tym opóźnieniem generowania względem siebie
         * statek gracza jest zdolny ulec destrukcji. Metoda bada poprawne przydzielanie stanu pola graSkończona = false, gdy
         * ilośćŻyć<=0. Poprzez manipulowanie stałymi można wygenerować stan, w którym to statki przeciwnika wykonają zbyt mało strzałów, lub/i
         * jest ich zbyt mało, by zagrozić naszemu statkowi.
         * UWAGA DLA PROGRAMISTY - na poczet tego testu zmieniono "zasięg" metody DodajPrzeciwnika z private na public
         * (metoda odpowiednio oflagowana w GalagaSilnik.cs)
         * UWAGA TECHNICZNA - METODA BARDZO OBCIĄŻAJĄCA!
        */
        [TestMethod]
        public void TestZniszczeniaStatkuGracza()
        {
            Silnik silnik = new Silnik();
            Random losowanie = new Random();
            const int ilośćPrzeciwników = 100; //dobrana wartość, dla której to statek zostanie zniszczony
            const int górnyZakresLosowości = 20; //dobrana wartość, dla której to statek zostanie zniszczony
            const int ilośćPrób = 2;
            int ilośćPoprawnychPrób = 0;

            silnik.Resetuj();
            for(int k = 0; k < ilośćPrób; k++)
            {
                for (int i = 0; i < ilośćPrzeciwników; i++)
                {
                    silnik.DodajPrzeciwnika(Convert.ToBoolean(losowanie.Next(0, 2)));
                    for (int j = losowanie.Next(0, górnyZakresLosowości); j > 0; j--)
                    {
                        silnik.WykonajKrok();
                    }
                }
                if (silnik.GraSkończona) ilośćPoprawnychPrób++;
            }
            Assert.AreEqual(ilośćPrób, ilośćPoprawnychPrób);
        }
        /* Metoda testowa badająca czy statki przeciwnika schodzą do finalnego pułapu Y=0.
         * Pętla while generuje kolejne ruchy statków.
         * Przydatne przy badaniu poprawności krzywej lotu statków.
         * UWAGA TECHNICZNA - METODA BARDZO OBCIĄŻAJĄCA!
         */
        [TestMethod]
        public void TestOpadaniaStatkówPrzeciwnikaDoYZero()
        {
            Silnik silnik = new Silnik();
            Random losowanie = new Random();
            bool spełnienieWarunku = false;
            const int ilośćStatków = 5;
            int ilośćPoprawnychWartości = 0;
            bool[] statkiKtóreOsiągnęłyPułap = new bool[ilośćStatków];

            silnik.Resetuj();
            for (int i = 0; i < ilośćStatków; i++) //przygotowanie statków do testów!
            {
                silnik.DodajPrzeciwnika(Convert.ToBoolean(losowanie.Next(0, 2))); //losowanie startu - lewa, lub prawa strona!
                statkiKtóreOsiągnęłyPułap[i] = false;
                for(int j = losowanie.Next(0, 2); j > 0; j--)
                {
                    silnik.WykonajKrok();
                }
            }
            while(spełnienieWarunku == false) //obsługa statków!
            {
                for(int i = 0; i < ilośćStatków; i++)
                {
                    if(silnik.ZwróćYPrzeciwnika(i) <= 0 && statkiKtóreOsiągnęłyPułap[i] == false)
                    {
                        ilośćPoprawnychWartości++;
                        statkiKtóreOsiągnęłyPułap[i] = true;
                        if (ilośćPoprawnychWartości == ilośćStatków)
                        {
                            spełnienieWarunku = true;
                        }
                    }
                    silnik.WykonajKrok();
                }
            }
            Assert.IsTrue(spełnienieWarunku);
        }
        /* Metoda testowa badająca czy statek gracza może wyjść za strefę gry.
         * W tej metodzie losuje się ruch w lewo, lub w prawo, a nastepnie bada się położenie.
         * Jeśli statek wyjdzie z przedziału <0, MaxXPlanszy>, to zostanie zgłoszony błąd.
         */
        [TestMethod]
        public void TestRuchówStatkuGraczaPoOsiX()
        {
            Silnik silnik = new Silnik();
            Random losowanie = new Random();
            int ilośćNiepoprawnegoWykroczenia = 0;
            const int ilośćLosowańKierunku = 100;
            const int górnyZakresLosowości = 25;

            silnik.Resetuj();
            silnik.PrzeskalujRozmiary(1920, 1080);
            for(int i = 0; i < ilośćLosowańKierunku; i++)
            {
                if(Convert.ToBoolean(losowanie.Next(0, 2))) 
                    silnik.UstawKlawiszStrzałkiWLewo();
                else 
                    silnik.UstawKlawiszStrzałkiWPrawo();

                for(int k = losowanie.Next(0, górnyZakresLosowości); k > 0; k--)
                {
                    silnik.WykonajKrok();
                    if(silnik.ZwróćXStatku() < 0 || silnik.ZwróćXStatku() > silnik.PrzestrzeńXpx)
                    {
                        ilośćNiepoprawnegoWykroczenia++;
                    }
                }
            }
            Assert.AreEqual(0, ilośćNiepoprawnegoWykroczenia);
        }
        /* Metoda do badania poprawnego zgłaszania wyjątku w momencie dodawania większej niż przewidywana ilości przeciwników.
         * W ogólności metoda bada czy silnik nie ma skłonności do akceptowania nadmiarowej ilości przeciwników, a przez to
         * odwoływania się do elementu wykraczającego poza zakres tablicy przeciwników.
         */
        [TestMethod]
        public void TestMaksymalnejIlościPrzeciwników()
        {
            Silnik silnik = new Silnik();
            int iloścPrób;
            Random losowanie = new Random();
            Exception wyjątek = null;

            silnik.Resetuj();
            iloścPrób = silnik.MaxLiczbaPrzeciwników + 5; //celowe wyjście poza maksymalny zakres ilości przeciwników!
            for (int i = 0; i < iloścPrób; i++)
            {
                silnik.DodajPrzeciwnika(Convert.ToBoolean(losowanie.Next(0, 2)));
                for(int k = losowanie.Next(0, 10); k > 0; k--)
                {
                    silnik.WykonajKrok();
                }
                try
                {
                    silnik.CzyPrzeciwnikIstnieje(i);
                }
                catch(Exception wyj)
                {
                    wyjątek = wyj; //łapanie oczekiwanego wyjątku wynikającego z nadmiarowego dodawania elementów.
                }
            }
            Assert.IsNotNull(wyjątek);
        }
    }
}
